@@ -1,4 +1,4 @@
-/**
+/*
  *  @internal
  * 
  *  LibAxl:  Another XML library 
@@ -30,12 +30,13 @@
  *          
  *      Postal address:
  *         Advanced Software Production Line, S.L.
- *         C/ Dr. Michavila N 14
- *         Coslada 28820 Madrid
+ *         Edificio Alius A, Oficina 102,
+ *         C/ Antonio Suarez Nº 10,
+ *         Alcalá de Henares 28802 Madrid
  *         Spain
  *
  *      Email address:
- *         info@aspl.es - http://fact.aspl.es
+ *         info@aspl.es - http://www.aspl.es/xml
  */
 #include <axl.h>
 
@@ -83,8 +84,9 @@
  *
  * <li>Have a <b>modular</b> design that allows to use only those
  * elements required by your software. At this moment it is provided a
- * base library (with implements XML 1.0) and an optional library that
- * provides additional features to support XML Namespaces 1.0.</li>
+ * base library (with implements XML 1.0) and optionals libraries that
+ * provides additional features to support XML Namespaces 1.0 and
+ * extended encodings.</li>
  *
  * <li>The other issue is that the Af-Arch client platform should be
  * easily embedded, so, a <b>small footprint</b> is a requirement. Currently LibAxl (89K) and LibAxl-Ns (9K)</li>
@@ -116,8 +118,10 @@
  *
  *  - <a href="http://www.w3.org/TR/REC-xml-names/">XML 1.0
  *  Namespaces</a> full support, through the additional component
- *  (libaxl-ns), allowing to produce xml applications that are
+ *  (<b>libaxl-ns</b>), allowing to produce xml applications that are
  *  XML Namespace aware. 
+ *
+ *  - Support for extended encodings (<b>libaxl-babel</b>), beyond default encoding utf-8. 
  * 
  * \section documentation Library Documentation
  *
@@ -127,6 +131,7 @@
  * - \ref axl_install
  * - \ref axl_manual
  * - \ref axl_api
+ * - \ref axl_knife_manual
  * 
  * \section contact_us Contact us
  * 
@@ -161,6 +166,7 @@
  *
  * - \ref validation
  * - \ref xml_namespace
+ * - \ref using_axl_babel
  *
  * <b>Section 4: Advanced topics </b><br>
  *
@@ -451,7 +457,7 @@
  *    doc = axl_doc_parse_from_file ("large.xml", error);
  *    if (doc == NULL) {
  *        axl_error_free (error);
- *        return false;
+ *        return axl_false;
  *    }
  *
  *    // DO SOME WORK WITH THE DOCUMENT HERE
@@ -462,7 +468,7 @@
  *    // cleanup axl library 
  *    axl_end ();
  *
- *    return true;
+ *    return axl_true;
  * }
  * \endcode
  *
@@ -660,16 +666,16 @@
  *	// parse gmovil file (an af-arch xml chunk) 
  *	doc = axl_doc_parse_from_file ("channel.xml", error); 
  *	if (doc == NULL) 
- *		return false;
+ *		return axl_false;
  *
  *	// parse af-arch DTD 
  *	dtd = axl_dtd_parse_from_file ("channel.dtd", error);
  *	if (dtd == NULL)
- *		return false;
+ *		return axl_false;
  *
  *	// perform DTD validation 
  *	if (! axl_dtd_validate (doc, dtd, error)) {
- *		return false;
+ *		return axl_false;
  *	}
  *
  *	// free doc reference 
@@ -678,10 +684,28 @@
  *      // free dtd reference
  *      axl_doc_free (dtd);
  * 
- *      return true;
+ *      return axl_true;
  * }
  * \endcode
  *
+ * Until now, we have seen how to check xml structure. But this do not
+ * cover xml node attributes. This is checked by using
+ * <b>&lt;!ATTLIST></b> declaration.
+ *
+ * In the case we have a node <b>testA</b> with two attribuets
+ * <b>attr1</b> and <b>attr2</b> the first one optional and the second
+ * one mandatory, we can declare something like:
+ *
+ * <div class="xml-doc">
+ * \code
+ * <!-- attribute validation for node testA -->
+ * <!ATTLIST testA 
+ *           attr1     CDATA   #IMPLIED
+ *           attr2     CDATA   #REQUIRED>
+ * \endcode
+ * </div>
+ * 
+ *
  * \section xml_namespace Enabling your software with XML Namespaces
  *
  * XML 1.0 initial design didn't take care about situations where
@@ -717,6 +741,66 @@
  * 
  * - \ref axl_ns_doc_module
  * - \ref axl_ns_node_module
+ *
+ * \section using_axl_babel Making your software to support other encodings than UTF-8
+ *
+ * Default axl library implementation (libaxl) assumes it will receive
+ * and produce UTF-8 content. 
+ * 
+ * Because the subset of characters that are used to properly parse
+ * XML content are located in the ASCII range, still valid UTF-8, but
+ * at same time valid in other encodings such ISO 646, some part of
+ * ISO 8859, Shift-JIS, EUC, or any other 7-bit, 8-bit, or mixed-width
+ * encoding which ensures that the characters of ASCII have their
+ * normal positions, width, and values (See section F. Autodetecting
+ * of Character Encodings at http://www.w3.org/TR/REC-xml/), causes
+ * the library to properly parse the content, even if it is not UTF-8.
+ *
+ * In many cases this is not important for you since your application
+ * do not care about content codification (such configuration files)
+ * or they are in UTF-8.
+ *
+ * However, this could present problems if you are handling different
+ * documents with several encoding types. The idea is to have an
+ * unified way to handle such different encoded documents, with a
+ * single, run-time encoding: UTF-8.
+ *
+ * <b>libaxl-babel</b> provides support to read content in supported
+ * codifications and translate it into UTF-8 at run-time (checking
+ * result to be valid UTF-8):
+ *
+ * \image html axl_babel_reading.png "Reading documents and handle them as they were in UTF-8"
+ *
+ * The library works as an extension that configures a set of handlers
+ * making the library to open XML documents and translating them into
+ * UTF-8 if required.
+ * 
+ * To activate the library, you must use \ref axl_babel_init at the
+ * begining of your application or library. Here is an example:
+ *
+ * \code
+ * // optional axlError declaration
+ * axlError * error;
+ * 
+ * // init axl babel 
+ * if (! axl_babel_init (&error)) {
+ *      printf ("Failed to start axl babel: %s...\n",
+ *              axl_error_get (error));
+ *      axl_error_free (error);
+ *      return axl_false;
+ * } 
+ * \endcode
+ * 
+ * Once done, every call to the base API (such \ref axl_doc_parse,
+ * \ref axl_doc_parse_from_file) will open the document as usual. It
+ * is not required to perform any additional special operation.
+ *
+ * It is not required to call to \ref axl_babel_finish on application
+ * exit. However, in the case you want to deactivate
+ * <b>libaxl-babel</b> but still keep on using axl base library, you
+ * can use \ref axl_babel_finish.
+ *
+ * See \ref axl_babel_init for currently supported formats.
  * 
  * \section reducing_foot_print How to reduce the library footprint 
  *
@@ -734,7 +818,7 @@
  * using <b>--axl-log-disable</b> as follow:
  *
  * \code
- * ./configure --axl-log-disable
+ * >> ./configure --axl-log-disable
  * \endcode
  *
  * According to our results, the library including the log to console
@@ -748,7 +832,7 @@
  * production environments doing the following once finished compilation process:
  *
  * \code
- * make install-strip
+ * >> make install-strip
  * \endcode
  *
  * According to our results, the library without log to console and
@@ -806,6 +890,11 @@
  *    - \ref axl_ns_node_module
  *
  *  </li>
+ *  <li><b>Axl BABEL API (required to support additional encoding formats): </b></li>
+ *
+ *    - \ref axl_babel
+ *
+ *  </li>
  *  <li><b>Error reporting and debugging functions (base library libaxl): </b></li>
  * 
  *    - \ref axl_error_module
@@ -817,6 +906,7 @@
  * - \ref axl_list_module
  * - \ref axl_list_cursor_module
  * - \ref axl_stack_module
+ * - \ref axl_binary_stack_module
  * - \ref axl_hash_module
  * - \ref axl_hash_cursor_module
  * - \ref axl_string_module 
@@ -826,7 +916,7 @@
  * 
  */
 
-/**
+/** 
  * @page axl_install Installing Axl library and using it
  *
  * \section intro Introduction
@@ -835,10 +925,28 @@
  * work on Microsoft Windows, GNU/Linux, *-BSD, GNU/Linux under
  * Amd64/Itanium and MacOS/X platforms. 
  *
+ * The library package is composed by the following items:
+ * 
+ * \image html axl-components.png "Axl library components"
+ * 
+ * - <b>libaxl</b>: base library implementing all XML functions, and
+ * common API used by extension libraries. It has no external
+ * dependencies.
+ *
+ * - <b>libaxl-ns</b>: optional library, built on top of libaxl, which
+ * provides Namespaces support. It depends on libaxl.
+ *
+ * - <b>libaxl-babel</b>: optional library, built on top of libaxl,
+ * which provides extended encoding support to defult utf-8.
+ *
+ * - <b>axl-knife</b>: command line tool, built on top of libaxl and
+ * libaxl-ns. It depends on the base library and the namespace
+ * library.
+ *
  * Here are a set of instructions to get the library compiled for your
  * platform:
  *
- * \section unix GNU/Linux (or any posix OS) installation instructions
+ * \section axl_on_unix GNU/Linux (or any posix OS) installation instructions
  *
  * First, download the package from the download section. Check <a
  * href="http://www.aspl.es/axl/doc.html">this section to know more
@@ -848,13 +956,23 @@
  * package. Here are the steps:
  *
  * \code
- *  bash: ~$ tar xzvf axl-0.2.1.b1984.g1985.tar.gz
- *  bash: ~$ cd axl-0.2.1.b1984.g1985/
- *  bash: ~/axl-0.2.1.b1984.g1985 $ ./configure
- *  bash: ~/axl-0.2.1.b1984.g1985 $ make
- *  bash: ~/axl-0.2.1.b1984.g1985 $ make install
+ *  >> tar xzvf axl-0.2.1.b1984.g1985.tar.gz
+ *  >> cd axl-0.2.1.b1984.g1985/
+ *  >> ./configure
+ *  >> make
+ *  >> make install
  * \endcode
  * 
+ * \section checking_axl Checking Axl Library compilation (regression test)
+ *
+ * Once finished, to check your axl build to properly work, get inside
+ * the test/ directory and run axl regression test (test_01):
+ * \code
+ *  >> cd test/
+ *  >> ./test_01
+ * \endcode
+ * 
+ * All test must execute properly to ensure the library is fully functional. 
  *
  * \section windows Windows installation instructions
  *
@@ -873,12 +991,17 @@
  * To check current support for your Axl Library you can execute:
  * 
  * \code
- *  bash: ~$ pkg-config --libs --cflags axl
+ *  >> pkg-config --libs --cflags axl
  * \endcode
  *
- * In the case you want give support for XML Namespaces, you must use:
+ * To give support for XML Namespaces (libaxl-ns), you must use:
  * \code
- *  bash: ~$ pkg-config --libs --cflags axl-ns
+ *  >> pkg-config --libs --cflags axl-ns
+ * \endcode
+ *
+ * To give support for extended encoding support (libaxl-babel), you must use:
+ * \code
+ *  >> pkg-config --libs --cflags axl-babel
  * \endcode
  *
  * To include support into your autotool checks (configure.ac/in
@@ -887,8 +1010,9 @@
  * PKG_CHECK_MODULES(LIBRARIES, axl) 
  * \endcode
  *
- * Again, add <b>axl-ns</b> to the previous instruction if you want
- * your package to also check for Axl Library Namespace support.
+ * Again, add <b>axl-ns</b> or <b>axl-babel</b> to the previous
+ * instruction if you want your package to also check for Axl Library
+ * Namespace support.
  * 
  * \section Including Axl Library headers
  *
@@ -901,6 +1025,11 @@
  * \code
  * #include <axl_ns.h>
  * \endcode
+ *
+ * To include babel support, you must use: 
+ * \code
+ * #include <axl_babel.h>
+ * \endcode
  */
 
 
@@ -922,13 +1051,13 @@
  *
  * However the API is provided for future usage.
  * 
- * @return The function returns true if it was properly
- * initialized or false if something fails.  if fails.
+ * @return The function returns \ref axl_true if it was properly
+ * initialized or \ref axl_false if something fails.  if fails.
  */
-bool axl_init ()
+axl_bool axl_init ()
 {
 	/* nothing to initialize dude */
-	return true;
+	return axl_true;
 }
 
 /** 
