@@ -24,7 +24,7 @@
  *
  * @return A \ref axlPointer reference.
  */
-#define INT_TO_PTR(integer)   ((axlPointer) (long) (integer))
+#define INT_TO_PTR(integer)   ((axlPointer)  ((int)integer))
 
 /**
  * @brief Allows to convert a pointer reference (\ref axlPointer),
@@ -37,7 +37,7 @@
  *
  * @return A int value.
  */
-#define PTR_TO_INT(ptr) ((int) (long) (ptr))
+#define PTR_TO_INT(ptr) ((int)  (ptr))
 
 /**
  * @brief Allows to get current platform configuration. This is used
