@@ -1,4 +1,4 @@
-/**
+/*
  *  LibAxl:  Another XML library
  *  Copyright (C) 2006 Advanced Software Production Line, S.L.
  *
@@ -28,12 +28,13 @@
  *          
  *      Postal address:
  *         Advanced Software Production Line, S.L.
- *         C/ Dr. Michavila N 14
- *         Coslada 28820 Madrid
+ *         Edificio Alius A, Oficina 102,
+ *         C/ Antonio Suarez Nº 10,
+ *         Alcalá de Henares 28802 Madrid
  *         Spain
  *
  *      Email address:
- *         info@aspl.es - http://fact.aspl.es
+ *         info@aspl.es - http://www.aspl.es/xml
  */
 #ifndef __AXL_DECL_H__
 #define __AXL_DECL_H__
@@ -49,10 +50,14 @@
 #  define __AXL_PRETTY_FUNCTION__ __PRETTY_FUNCTION__
 #  define __AXL_LINE__            __LINE__
 #  define __AXL_FILE__            __FILE__
+#elif defined(_MSC_VER)
+#  define __AXL_PRETTY_FUNCTION__ __FUNCDNAME__
+#  define __AXL_LINE__            __LINE__
+#  define __AXL_FILE__            __FILE__
 #else
-/* non gnu compiler */
+/* unknown compiler */
 #define __AXL_PRETTY_FUNCTION__ ""
-#define __AXL_LINE__            ""
+#define __AXL_LINE__            0
 #define __AXL_FILE__            ""
 #endif
 
@@ -594,7 +599,7 @@ typedef struct _axlPI        axlPI;
  *     printf ("Parse error: code=%d, message=%s\n", 
  *             axl_error_get_code (error), axl_error_get (error));
  *     axl_error_free (error);
- *     return false;
+ *     return axl_false;
  * }
  *
  * // beyond this point, it is not required to do
@@ -612,24 +617,22 @@ typedef struct _axlError  axlError;
 typedef struct _axlStream axlStream;
 
 /** 
- * @brief (DEPRECATED use \ref true) Type definition to represent a
- * boolean true value, that is equal to 1.
+ * @brief (DEPRECATED use \ref axl_true) Type definition to represent a boolean true value, that is equal to 1.
  */
-#define AXL_TRUE  (1)
+#define AXL_TRUE  (axl_true)
 
 /** 
- * @brief (DEPRECATED use \ref false) Type definition to represent a
- * boolean false value, that is equal to 0.
+ * @brief (DEPRECATED use \ref axl_false) Type definition to represent a* boolean false value, that is equal to 0.
  */
-#define AXL_FALSE (0)
+#define AXL_FALSE (axl_false)
 
 /** 
  * @brief Alias declaration to bind the <i>int</i> to the <b>boolean</b>
  * concept (TRUE / FALSE states) (DEPRECATED).
  *
  * This is mainly used to emphasize that some integer values that
- * returns some function must be considered to be \ref true or \ref
- * false, that represents the boolean TRUE and FALSE values.
+ * returns some function must be considered to be \ref axl_true or \ref
+ * axl_false, that represents the boolean TRUE and FALSE values.
  *
  * This allows to perform boolean comparations using structure
  * controls like if, while, but also making a differenciation about
@@ -640,25 +643,60 @@ typedef struct _axlStream axlStream;
 typedef int aboolean;
 
 /**
- * @internal
- * 
- * Internal definition to retain compatibility with programs that
- * define the bool value but the compiler doesn't provide it.
+ * @brief Bool definition for the Axl library. This type built on top
+ * of <b>int</b> is used along with \ref axl_false and \ref axl_true
+ * to model those API functions and attributes that returns or receive
+ * a boolean state. 
  */
-#ifndef __cplusplus
-typedef int bool;
+typedef int axl_bool;
 
 /** 
- * @brief Common definition to have false value (which is defined to 0
- * integer value).
+ * @brief Common definition to have false (\ref axl_false) value (which is defined to 0 integer value).
  */
-#define false (0)
+#define axl_false ((int)0)
 /** 
- * @brief Common definition to have true value (which is defined to 1
- * integer value).
+ * @brief Common definition to have true (\ref axl_true) value (which is defined to 1 integer value).
  */
-#define true  (1)
-#endif /* __cplusplus */
+#define axl_true  ((int)1)
+
+/**
+ * @brief Boolean, deprecated, compatibility mode provided for those
+ * applications and libraries built on top of Axl Library or any
+ * library on top of it (like Vortex Library) to support people from C
+ * user space using bool type definition and its corresponding values
+ * false and true as they was available before 0.5.5 release.
+ *
+ * Definitions provided by this compatibility mode are:
+ *
+ * - \ref bool
+ * - \ref true
+ * - \ref false
+ */
+#if !defined(__cplusplus) && !defined(__bool_true_false_are_defined) && !defined(__axl_disable_broken_bool_def__)
+/**
+ * @brief <b>DEPRECATED:</b> boolean definition provided to support
+ * applications and libraries using this type as defined by Axl
+ * Library before 0.5.5 release. Do use this for newly written code.
+ */
+typedef int bool;
+
+/**
+ * @brief <b>DEPRECATED:</b> boolean false definition provided to
+ * support applications and libraries using this type as defined by
+ * Axl Library before 0.5.5 release. Do use this for newly written
+ * code.
+ */ 
+#define false axl_false
+
+/**
+ * @brief <b>DEPRECATED:</b> boolean true definition provided to
+ * support applications and libraries using this type as defined by
+ * Axl Library before 0.5.5 release. Do use this for newly written
+ * code.
+ */ 
+#define true axl_true
+#endif
+
 
 /** 
  * @brief Pointer to any structure definition. It should be required
@@ -687,6 +725,16 @@ typedef struct _axlListCursor axlListCursor;
  */
 typedef struct _axlStack axlStack;
 
+/** 
+ * @brief Compact binary state representation stack.
+ *
+ * This data structure allows to store binary/boolean values in an
+ * efficient way.
+ *
+ * See \ref axl_binary_stack_new for more information.
+ */
+typedef struct _axlBinaryStack axlBinaryStack;
+
 /** 
  * @brief \ref axlHash definition, a hash table to store key indexed
  * values.
@@ -738,6 +786,65 @@ typedef void (*axlDestroyFunc) (axlPointer ptr);
  */
 typedef char * (*axlStreamAlloc) (int size, axlPointer data);
 
+/** 
+ * @brief Handler definition for the set of functions that allows to
+ * translate content into a particular format back to utf-8, which is
+ * the default format used for internal data stored by Axl. 
+ * 
+ * @param source The source content to be decoded into utf-8. 
+ *
+ * @param source_size The size of the source content to be decoded.
+ *
+ * @param source_encoding The encoding of the source. 
+ *
+ * @param output The output of the decode operation. This buffer is
+ * memory allocated by the caller. The size to be written is limited
+ * by the following parameter.
+ *
+ * @param output_size The size of the output produced (in terms of
+ * octects not utf-8 logical units).
+ *
+ * @param remain_source_index Reference where the last index of valid
+ * input still pending to be process. Especially useful under
+ * situation where the decode function returns 2.
+ * 
+ * @return The handler must return 1 if the operation was completed, 2
+ * if the operation was completed but not enough size was found on
+ * output buffer to store the content or 0 if the function fails.
+ */
+typedef int (*axlStreamDecode) (const char * source, int source_size,
+				const char * source_encoding,
+				char * output, int output_size, 
+				int * output_converted,
+				int * remain_source_index,
+				axlPointer user_data);
+
+/** 
+ * @brief A handler definition used by axl stream API to define the
+ * set of functions that can be used to check content read from a file
+ * into the axl stream buffer. The initial intention was to allow axl
+ * babel to install an utf-8 content checked if an xml entity in utf-8
+ * was detected but it allows more features.
+ *
+ * This handler is configured using \ref axl_stream_setup_check.
+ * 
+ * @param source The source to check.
+ *
+ * @param source_size The size of the source to check.
+ *
+ * @param source_encoding The source encoding found.
+ *
+ * @param user_data User defined pointer 
+ * 
+ * @return 
+ */
+typedef int (*axlStreamContentCheck) (const char  * source, 
+				      int           source_size,
+				      const char  * source_encoding,
+				      axlPointer    user_data,
+				      axlError   ** error);
+				      
+
 /** 
  * @brief Axl debug levels.
  * 
@@ -766,8 +873,12 @@ typedef enum {
 	AXL_LEVEL_CRITICAL}  
 AxlDebugLevel;
 
+axlPointer  axl_calloc(size_t count, size_t size);
+
+axlPointer  axl_realloc(axlPointer ref, size_t size);
+
 /** 
- * @brief Support macro to allocate memory using the calloc function,
+ * @brief Support macro to allocate memory using axl_calloc function,
  * making a casting and using the sizeof keyword.
  *
  * @param type The type to allocate
@@ -775,16 +886,12 @@ AxlDebugLevel;
  * 
  * @return A newly allocated pointer.
  */
-#define axl_new(type, count) (type *) calloc (count, sizeof (type))
+#define axl_new(type, count) (type *) axl_calloc (count, sizeof (type))
 
-/** 
- * @brief Free the provided pointer that was allocated using \ref
- * axl_new.
- * 
- * @param ref The reference to deallocate.
- */
 void    axl_free(axlPointer ref);
 
+
+
 /** 
  * @brief Allows to check a condition and return if it is not meet.
  * 
@@ -824,10 +931,10 @@ axl_stream_consume_white_spaces (stream)
  * 
  * @param str The string to check for emptyness.
  * 
- * @return Returns true if the string is empty and false if
+ * @return Returns axl_true if the string is empty and axl_false if
  * not.
  */
-#define AXL_IS_STR_EMPTY(str) (((str == NULL) || strlen (str) == 0) ? true : false)
+#define AXL_IS_STR_EMPTY(str) (((str == NULL) || strlen (str) == 0) ? axl_true : axl_false)
 
 /** 
  * @internal
@@ -853,12 +960,12 @@ axl_stream_consume_white_spaces (stream)
 i = 0;\
 while (s1 [i] != 0 && s2 [i] != 0) {\
      if (s1 [i] != s2 [i])\
-	return false;\
+	return axl_false;\
      i++;\
      if (i == size)\
-	return true;\
+	return axl_true;\
 }\
-return false
+return axl_false
 
 /** 
  * @brief Allows to configure how is performed the iteration other the xml document.
@@ -903,8 +1010,8 @@ typedef enum {
  * (\ref axl_doc_iterate).
  *
  * The function returns a boolean value to signal the library to stop
- * iterating over the XML structure if \ref false is returned. So, to
- * continue the iteration, you must always return \ref true.
+ * iterating over the XML structure if \ref axl_false is returned. So, to
+ * continue the iteration, you must always return \ref axl_true.
  * 
  * @param node The node found inside the document.
  *
@@ -924,10 +1031,10 @@ typedef enum {
  * @param ptr A user defined pointer that the user provided at \ref
  * axl_doc_iterate.
  * 
- * @return The callback must return false in the case the iteration
- * must be stopped. Otherwise, true must be returned.
+ * @return The callback must return axl_false in the case the iteration
+ * must be stopped. Otherwise, axl_true must be returned.
  */
-typedef bool (*axlIterationFunc) (axlNode * node, axlNode * parent, axlDoc * doc, bool * was_removed, axlPointer ptr);
+typedef axl_bool (*axlIterationFunc) (axlNode * node, axlNode * parent, axlDoc * doc, axl_bool * was_removed, axlPointer ptr);
 
 /** 
  * @brief Axl iteration function definition (with two user defined
@@ -943,9 +1050,9 @@ typedef bool (*axlIterationFunc) (axlNode * node, axlNode * parent, axlDoc * doc
  * optional user defined pointer provided at the function calling
  * (\ref axl_doc_iterate_full).
  *
- * The function returns a boolean value to signal the library to stop
- * iterating over the XML structure if \ref false is returned. So, to
- * continue the iteration, you must always return \ref true.
+ * The function returns a axl_boolean value to signal the library to stop
+ * iterating over the XML structure if \ref axl_false is returned. So, to
+ * continue the iteration, you must always return \ref axl_true.
  * 
  * @param node The node found inside the document.
  *
@@ -968,10 +1075,10 @@ typedef bool (*axlIterationFunc) (axlNode * node, axlNode * parent, axlDoc * doc
  * @param ptr2 Second user defined pointer that the user provided at
  * \ref axl_doc_iterate_full.
  * 
- * @return The callback must return false in the case the iteration
- * must be stopped. Otherwise, true must be returned.
+ * @return The callback must return axl_false in the case the iteration
+ * must be stopped. Otherwise, axl_true must be returned.
  */
-typedef bool (*axlIterationFunc2) (axlNode * node, axlNode * parent, axlDoc * doc, bool * was_removed, axlPointer ptr, axlPointer ptr2);
+typedef axl_bool (*axlIterationFunc2) (axlNode * node, axlNode * parent, axlDoc * doc, axl_bool * was_removed, axlPointer ptr, axlPointer ptr2);
 
 /** 
  * @brief Defines a signature for a set of function that are used to
@@ -998,10 +1105,10 @@ typedef axlPointer (*axlDuplicateFunc) (axlPointer ptr);
  * @param data A pointer to a user defined data that is received at
  * the lookup function and passed to this handler.
  * 
- * @return The function should return true (found). Otherwise, false
+ * @return The function should return axl_true (found). Otherwise, axl_false
  * must be returned to keep on searching.
  */
-typedef bool (*axlLookupFunc) (axlPointer ptr, axlPointer data);
+typedef axl_bool (*axlLookupFunc) (axlPointer ptr, axlPointer data);
 
 /** 
  * @brief Hashing function used by the axl hash module to implement
@@ -1025,19 +1132,19 @@ typedef unsigned int (*axlHashFunc) (axlPointer key);
  * 
  * The function receives the item found (key and data values) as well
  * as a user defined pointer also defined at \ref
- * axl_hash_foreach. The function must return \ref true (<i>"item
+ * axl_hash_foreach. The function must return \ref axl_true (<i>"item
  * found"</i>) to make the search to stop. In the case a full
  * iteration over all items inside the hash is required, the function
- * must always return \ref false.
+ * must always return \ref axl_false.
  * 
  * @param key The key for the item stored.
  * @param data The data associated to the key found
  * @param user_data User defined data that was provided to the axl_hash_foreach function.
  * 
- * @return \ref true to make the foreach process to stop. \ref false
+ * @return \ref axl_true to make the foreach process to stop. \ref axl_false
  * to make the process to continue.
  */
-typedef bool (* axlHashForeachFunc) (axlPointer key, axlPointer data, axlPointer user_data);
+typedef axl_bool (* axlHashForeachFunc) (axlPointer key, axlPointer data, axlPointer user_data);
 
 /** 
  * @brief Foreach function signature used to represent the set of
@@ -1045,20 +1152,20 @@ typedef bool (* axlHashForeachFunc) (axlPointer key, axlPointer data, axlPointer
  * 
  * The function receives the item found (key and data values) as well
  * as two user defined pointers also defined at \ref
- * axl_hash_foreach2. The function must return \ref true (<i>"item
+ * axl_hash_foreach2. The function must return \ref axl_true (<i>"item
  * found"</i>) to make the search to stop. In the case a full
  * iteration over all items inside the hash is required, the function
- * must always return \ref false.
+ * must always return \ref axl_false.
  * 
  * @param key The key for the item stored.
  * @param data The data associated to the key found
  * @param user_data User defined data that was provided to the axl_hash_foreach2 function.
  * @param user_data2 Second User defined data that was provided to the axl_hash_foreach2 function.
  * 
- * @return \ref true to make the foreach process to stop. \ref false
+ * @return \ref axl_true to make the foreach process to stop. \ref axl_false
  * to make the process to continue.
  */
-typedef bool (* axlHashForeachFunc2) (axlPointer key, axlPointer data, axlPointer user_data, axlPointer user_data2);
+typedef axl_bool (* axlHashForeachFunc2) (axlPointer key, axlPointer data, axlPointer user_data, axlPointer user_data2);
 
 /** 
  * @brief Foreach function signature used to represent the set of
@@ -1066,10 +1173,10 @@ typedef bool (* axlHashForeachFunc2) (axlPointer key, axlPointer data, axlPointe
  * 
  * The function receives the item found (key and data values) as well
  * as tree user defined pointers also defined at \ref
- * axl_hash_foreach3. The function must return \ref true (<i>"item
+ * axl_hash_foreach3. The function must return \ref axl_true (<i>"item
  * found"</i>) to make the search to stop. In the case a full
  * iteration over all items inside the hash is required, the function
- * must always return \ref false.
+ * must always return \ref axl_false.
  * 
  * @param key The key for the item stored.
  *
@@ -1084,10 +1191,10 @@ typedef bool (* axlHashForeachFunc2) (axlPointer key, axlPointer data, axlPointe
  * @param user_data3 Third User defined data that was provided to the
  * axl_hash_foreach3 function.
  * 
- * @return \ref true to make the foreach process to stop. \ref false
+ * @return \ref axl_true to make the foreach process to stop. \ref axl_false
  * to make the process to continue.
  */
-typedef bool (* axlHashForeachFunc3) (axlPointer key, axlPointer data, axlPointer user_data, axlPointer user_data2, axlPointer user_data3);
+typedef axl_bool (* axlHashForeachFunc3) (axlPointer key, axlPointer data, axlPointer user_data, axlPointer user_data2, axlPointer user_data3);
 
 /**
  * @brief Foreach function signature used to represent the set of
@@ -1095,10 +1202,10 @@ typedef bool (* axlHashForeachFunc3) (axlPointer key, axlPointer data, axlPointe
  * 
  * The function receives the item found (key and data values) as well
  * as tree user defined pointers also defined at \ref
- * axl_hash_foreach4. The function must return \ref true (<i>"item
+ * axl_hash_foreach4. The function must return \ref axl_true (<i>"item
  * found"</i>) to make the search to stop. In the case a full
  * iteration over all items inside the hash is required, the function
- * must always return \ref false.
+ * must always return \ref axl_false.
  * 
  * @param key The key for the item stored.
  *
@@ -1116,10 +1223,10 @@ typedef bool (* axlHashForeachFunc3) (axlPointer key, axlPointer data, axlPointe
  * @param user_data4 Forth User defined data that was provided to the
  * axl_hash_foreach4 function.
  * 
- * @return \ref true to make the foreach process to stop. \ref false
+ * @return \ref axl_true to make the foreach process to stop. \ref axl_false
  * to make the process to continue.
  */
-typedef bool (* axlHashForeachFunc4) (axlPointer key, axlPointer data, axlPointer user_data, axlPointer user_data2, axlPointer user_data3, axlPointer user_data4);
+typedef axl_bool (* axlHashForeachFunc4) (axlPointer key, axlPointer data, axlPointer user_data, axlPointer user_data2, axlPointer user_data3, axlPointer user_data4);
 
 /** 
  * @brief Function handler definition for to allowing copying items at
@@ -1167,10 +1274,10 @@ typedef axlPointer (*axlHashItemCopy) (axlPointer key, axlDestroyFunc key_destro
  * @param user_data2 A second reference to a user defined pointer
  * passed to \ref axl_stack_foreach.
  * 
- * @return \ref true to make the foreach process to stop. \ref false
+ * @return \ref axl_true to make the foreach process to stop. \ref axl_false
  * to make the process to continue.
  */
-typedef bool (* axlStackForeach2) (axlPointer stack_data, axlPointer user_data, axlPointer user_data2);
+typedef axl_bool (* axlStackForeach2) (axlPointer stack_data, axlPointer user_data, axlPointer user_data2);
 
 /** 
  * @brief Foreach function handler used at \ref axl_stack_foreach3
@@ -1191,10 +1298,10 @@ typedef bool (* axlStackForeach2) (axlPointer stack_data, axlPointer user_data,
  * @param user_data3 Third reference to a user defined pointer passed
  * to \ref axl_stack_foreach3.
  * 
- * @return \ref true to make the foreach process to stop. \ref false
+ * @return \ref axl_true to make the foreach process to stop. \ref axl_false
  * to make the process to continue.
  */
-typedef bool (* axlStackForeach3) (axlPointer stack_data, axlPointer user_data, axlPointer user_data2, axlPointer user_data3);
+typedef axl_bool (* axlStackForeach3) (axlPointer stack_data, axlPointer user_data, axlPointer user_data2, axlPointer user_data3);
 
 /** 
  * @brief Foreach function used by \ref axl_node_attr_foreach function.
@@ -1210,10 +1317,10 @@ typedef bool (* axlStackForeach3) (axlPointer stack_data, axlPointer user_data,
  * axl_node_attr_foreach.
  *
  * @return The foreach function can stop the process at a particular
- * attribute by returning \ref true ("item found"). To iterate all
- * attributes return \ref false.
+ * attribute by returning \ref axl_true ("item found"). To iterate all
+ * attributes return \ref axl_false.
  */
-typedef bool (* axlNodeAttrForeachFunc) (const char * key, const char * value, axlPointer data, axlPointer data2);
+typedef axl_bool (* axlNodeAttrForeachFunc) (const char * key, const char * value, axlPointer data, axlPointer data2);
 
 /** 
  * @brief Entity resolver function used by the library to translate
@@ -1231,6 +1338,48 @@ typedef bool (* axlNodeAttrForeachFunc) (const char * key, const char * value, a
  */
 typedef const char * (* axlDtdEntityResolver) (const char * entityName, axlPointer data);
 
+/** 
+ * @brief Handler definition for the set of functions that allows to
+ * detect codification found at the document being opened by the
+ * \ref axlStream reference provided.
+ * 
+ * @param stream The stream where the detection will be implemented.
+ *
+ * @param detected A reference to the codification detected or NULL if
+ * nothing clearly detected. For example (ascii, iso-8859) but still
+ * not enough information.
+ *
+ * @param user_data A reference to user-defined data. This value was
+ * configured at \ref axl_doc_set_detect_codification_func.
+ * 
+ * @return axl_true if the detection was implemented properly, otherse
+ * axl_false is returned. The handler could return axl_true and no
+ * codification be clearly detected.
+ */
+typedef axl_bool (* axlDocDetectCodification) (axlStream * stream, char ** detected, axlPointer user_data, axlError ** error);
+
+/** 
+ * @brief Handler definition for the set of functions that allows to
+ * finally configure codification to be used for the provided stream.
+ * 
+ * @param stream A reference to the stream to be configured.
+ *
+ * @param encoding A reference to the encoding detected. It could be
+ * NULL.
+ *
+ * @param detected_encoding A reference to the detected encoding (a
+ * value provided by the \ref axlDocDetectCodification if defined).
+ * 
+ * @param user_data A reference to user defined data.
+ *
+ * @param error An optional error that will be filled in the case an
+ * error is found.
+ * 
+ * @return axl_true if the configuration operation was done, otherwise
+ * axl_false is returned.
+ */
+typedef axl_bool (* axlDocConfigureCodification) (axlStream * stream, const char * encoding, const char * detected_encoding, axlPointer user_data, axlError ** error);
+
 /* @} */
 #endif
 
