@@ -28,12 +28,13 @@
  *          
  *      Postal address:
  *         Advanced Software Production Line, S.L.
- *         C/ Dr. Michavila N 14
- *         Coslada 28820 Madrid
+ *         Edificio Alius A, Oficina 102,
+ *         C/ Antonio Suarez Nº 10,
+ *         Alcalá de Henares 28802 Madrid
  *         Spain
  *
  *      Email address:
- *         info@aspl.es - http://fact.aspl.es
+ *         info@aspl.es - http://www.aspl.es/xml
  */
 
 #include <axl_decl.h>
@@ -272,6 +273,23 @@ axlList * axl_list_new    (axlEqualFunc are_equal, axlDestroyFunc destroy_data)
 	return result;
 }
 
+/**
+ * @brief Allows to reconfigure the destroy function to be used on
+ * this list. This function can be useful to disable the destroy
+ * function associated to the list. 
+ *
+ * @param list The list to reconfigure or disable (NULL) the destroy function.
+ *
+ * @param destroy_data The reference to the destroy function to be
+ * used or NULL to disable it.
+ */
+void       axl_list_set_destroy_func (axlList * list, axlDestroyFunc destroy_data)
+{
+	axl_return_if_fail (list);
+	list->destroy_data = destroy_data;
+	return;
+}
+
 /** 
  * @brief Allows to copy the provided list, returning a newly
  * allocated structure. 
@@ -284,6 +302,11 @@ axlList * axl_list_new    (axlEqualFunc are_equal, axlDestroyFunc destroy_data)
  * This handler is optional, and in the case it is not provided, the
  * list returned will be a copy having reference to the content
  * stored.
+ *
+ * NOTE: the function also copy the destroy and equal function
+ * configured at \ref axl_list_new. If you want to disable destroy
+ * function on the copy returned you must use \ref
+ * axl_list_set_destroy_func, passing NULL as destroy handler.
  * 
  * @param list The list to copy.
  * @param func The duplication function used to perform a deep copy (optional handler).
@@ -405,6 +428,13 @@ int        axl_list_equal_int    (axlPointer a, axlPointer b)
  * @param b Second item.
  * 
  * @return The function always return 1.
+ *
+ * NOTE: If you use this function and your intention is to remove
+ * items (without calling to axl_list_free) you must use \ref
+ * axl_list_remove_ptr or \ref axl_list_unlink_ptr since \ref
+ * axl_list_remove and \ref axl_list_unlink relays on the equal
+ * function to find and remove the item. Because this function never
+ * return 0, the item is never removed.
  */
 int        axl_list_always_return_1 (axlPointer a, axlPointer b)
 {
@@ -422,6 +452,11 @@ int        axl_list_always_return_1 (axlPointer a, axlPointer b)
  *
  * @param pointer The pointer to store.
  *
+ * NOTE: The function uses the equal function defined at \ref
+ * axl_list_new. If the function shows that the item to be added is
+ * already added (because the equal function returns 0, then the item
+ * won't be added.
+ *
  */
 void      axl_list_add    (axlList * list, axlPointer pointer)
 {
@@ -674,8 +709,8 @@ void       axl_list_append  (axlList * list, axlPointer pointer)
 
 /** 
  * @internal Internal list lookup using a linear search, checking all
- * items inside the list withtout taking into considerations hints
- * provided by equal function.
+ * items inside the list taking into considerations hints provided by
+ * equal function.
  * 
  * @param list The list where the linear search will be performed.
  * @param pointer The pointer that is being looked up.
@@ -707,6 +742,38 @@ axlListNode * axl_list_internal_linear_lookup (axlList    * list,
 	return NULL;
 }
 
+/** 
+ * @internal Internal list lookup using a linear search, checking all
+ * items inside the list withtout taking into considerations hints
+ * provided by equal function (search by pointer).
+ * 
+ * @param list The list where the linear search will be performed.
+ * @param pointer The pointer that is being looked up.
+ * 
+ * @return A reference to the internal axl list node containing the
+ * pointer.
+ */
+axlListNode * axl_list_internal_linear_lookup_ptr (axlList    * list, 
+						   axlPointer    pointer)
+{
+	axlListNode * node;
+
+	axl_return_val_if_fail (list, NULL);
+	axl_return_val_if_fail (pointer, NULL);
+
+	/* complex case */
+	node  = list->first_node;
+
+	/* lookup */
+	while (node && node->data != pointer) {
+		/* the node should be after this one */
+		node = node->next;
+	} /* end while */
+
+	/* return current result */
+	return node;
+}
+
 /** 
  * @internal
  * @brief Internal lookup function to locate the axlListNode that contains the pointer.
@@ -810,7 +877,7 @@ axlListNode * axl_list_internal_get_nth (axlList * list, int position)
 
 /* remove the selected node */
 void __axl_list_common_remove_selected_node (axlList * list, axlListNode * node, 
-					     bool alsoRemove)
+					     axl_bool alsoRemove)
 {
 	axlPointer pointer;
 
@@ -860,8 +927,10 @@ void __axl_list_common_remove_selected_node (axlList * list, axlListNode * node,
  * @param pointer The pointer to remove
  *
  * @param alsoRemove Also call to destroy function.
+ *
+ * @param byPtr Makes the linear search to be done by pointer.
  */
-void     axl_list_common_remove (axlList * list, axlPointer pointer, bool alsoRemove)
+void     axl_list_common_remove (axlList * list, axlPointer pointer, axl_bool alsoRemove, axl_bool byPtr)
 {
 	axlListNode * node;
 
@@ -869,7 +938,10 @@ void     axl_list_common_remove (axlList * list, axlPointer pointer, bool alsoRe
 	axl_return_if_fail (pointer);
 
 	/* complex case */
-	node  = axl_list_internal_linear_lookup (list, pointer);
+	if (byPtr)
+		node = axl_list_internal_linear_lookup_ptr (list, pointer);
+	else
+		node  = axl_list_internal_linear_lookup (list, pointer);
 	if (node == NULL) {
 		__axl_log (LOG_DOMAIN, AXL_LEVEL_DEBUG, "unable to find item by pointer (0x%x)",
 			   pointer);
@@ -897,18 +969,47 @@ void     axl_list_common_remove (axlList * list, axlPointer pointer, bool alsoRe
  *
  * @param list The list where the removal operation will be performed.
  * @param pointer The pointer where the 
+ *
+ * NOTE: The function uses the current equal function configured. A
+ * not properly configured equal function will make this function to
+ * not remove the item selected. If you are trying to remove by
+ * pointer, you can use \ref axl_list_remove_ptr.
  */
 void      axl_list_remove (axlList * list, axlPointer pointer)
 {
 
-	__axl_log (LOG_DOMAIN, AXL_LEVEL_DEBUG, "attempting to remove pointer: 0x%x", pointer);
-
 	/* perform a complete removing */
-	axl_list_common_remove (list, pointer, true);
+	axl_list_common_remove (list, pointer, 
+				/* alsoRemove */
+				axl_true, 
+				/* byPtr */ 
+				axl_false);
 
 	return;
 }
 
+/** 
+ * @brief Allows to remove the provided pointer from the list (calling
+ * to the destroy function defined).
+ *
+ * Unlike \ref axl_list_remove, which removes the element selected by
+ * using the equal function configured at \ref axl_list_new, this
+ * function allows to perform a remove operation by pointer.
+ * 
+ * @param list The list on which the operation is performed.
+ *
+ * @param pointer The pointer to remove from the list.
+ */
+void axl_list_remove_ptr (axlList * list, axlPointer pointer)
+{
+	/* perform a complete removing */
+	axl_list_common_remove (list, pointer, 
+				/* alsoRemove */
+				axl_true, 
+				/* byPtr */ 
+				axl_true);
+}
+
 /** 
  * @brief Removes the given pointer from the list, without calling the
  * destroy function, even when it is configured.
@@ -920,11 +1021,40 @@ void      axl_list_remove (axlList * list, axlPointer pointer)
 void       axl_list_unlink (axlList * list, axlPointer pointer)
 {
 	/* perform a complete removing */
-	axl_list_common_remove (list, pointer, false);
+	axl_list_common_remove (list, pointer, 
+				/* alsoRemove */
+				axl_false, 
+				/* byPtr */ 
+				axl_false);
 
 	return;
 }
 
+/** 
+ * @brief Allows to remove the provided item from the axl list
+ * withtout using the equal function provided (remove by pointer) and
+ * without calling to the associated destroy function.
+ * 
+ * @param list The list where the operation will be implemented.
+ *
+ * @param pointer the pointer to remove from the list without calling
+ * to the destroy function.
+ */
+void axl_list_unlink_ptr (axlList * list, axlPointer pointer)
+{
+
+	/* perform an unlink operation, without using equal
+	 * function */
+	/* perform a complete removing */
+	axl_list_common_remove (list, pointer, 
+				/* alsoRemove */
+				axl_false, 
+				/* byPtr */ 
+				axl_true);
+	
+	return;
+}
+
 /** 
  * @brief Allows to remove the first element, calling to the destroy
  * function if defined.
@@ -941,7 +1071,7 @@ void       axl_list_remove_first (axlList * list)
 		return;
 
 	/* remove the selected node */
-	__axl_list_common_remove_selected_node (list, list->first_node, true);
+	__axl_list_common_remove_selected_node (list, list->first_node, axl_true);
 
 	return;
 }
@@ -962,7 +1092,7 @@ void       axl_list_unlink_first (axlList * list)
 		return;
 
 	/* remove the selected node */
-	__axl_list_common_remove_selected_node (list, list->first_node, false);
+	__axl_list_common_remove_selected_node (list, list->first_node, axl_false);
 
 	return;
 }
@@ -982,7 +1112,7 @@ void       axl_list_remove_last (axlList * list)
 		return;
 
 	/* remove the selected node */
-	__axl_list_common_remove_selected_node (list, list->last_node, true);
+	__axl_list_common_remove_selected_node (list, list->last_node, axl_true);
 
 	return;
 }
@@ -1002,7 +1132,7 @@ void       axl_list_unlink_last (axlList * list)
 		return;
 
 	/* remove the selected node */
-	__axl_list_common_remove_selected_node (list, list->last_node, false);
+	__axl_list_common_remove_selected_node (list, list->last_node, axl_false);
 
 	return;
 }
@@ -1015,18 +1145,18 @@ void       axl_list_unlink_last (axlList * list)
  *
  * @param pointer The pointer to lookup.
  * 
- * @return \ref true if the element is stored on the list,
- * otherwise false is returned. The function will fail to lookup
+ * @return \ref axl_true if the element is stored on the list,
+ * otherwise axl_false is returned. The function will fail to lookup
  * if a NULL reference is received, either the list or the pointer.
  */
-bool          axl_list_exists (axlList * list, axlPointer pointer)
+axl_bool          axl_list_exists (axlList * list, axlPointer pointer)
 {
-	axl_return_val_if_fail (list, false);
-	axl_return_val_if_fail (pointer, false);
+	axl_return_val_if_fail (list, axl_false);
+	axl_return_val_if_fail (pointer, axl_false);
 
 	if (axl_list_internal_lookup (list, pointer) != NULL)
-		return true;
-	return false;
+		return axl_true;
+	return axl_false;
 }
 
 /** 
@@ -1035,12 +1165,12 @@ bool          axl_list_exists (axlList * list, axlPointer pointer)
  * 
  * @param list The list to check for emptyness.
  * 
- * @return true if the list is empty, false if not. The function
- * return false in the case a null reference is provided.
+ * @return axl_true if the list is empty, axl_false if not. The function
+ * return axl_false in the case a null reference is provided.
  */
-bool       axl_list_is_empty  (axlList * list)
+axl_bool       axl_list_is_empty  (axlList * list)
 {
-	axl_return_val_if_fail (list, false);
+	axl_return_val_if_fail (list, axl_false);
 
 	/* check if the first node is defined */
 	return (list->first_node == NULL);
@@ -1053,19 +1183,19 @@ bool       axl_list_is_empty  (axlList * list)
  * @param pointer The pointer to check.
  * @param position The position where is expected to find the pointer.
  * 
- * @return true if the given data, referenced by the pointer, is
+ * @return axl_true if the given data, referenced by the pointer, is
  * stored on the given position.
  */
-bool           axl_list_exists_at (axlList * list, axlPointer pointer, int position)
+axl_bool           axl_list_exists_at (axlList * list, axlPointer pointer, int position)
 {
 	axlListNode * node;
 
 	node = axl_list_internal_get_nth (list, position);
 	if (node != NULL) {
 		if (! list->are_equal (node->data, pointer))
-			return true;
+			return axl_true;
 	}
-	return false;
+	return axl_false;
 }
 
 /** 
@@ -1137,17 +1267,17 @@ axlPointer axl_list_get_nth   (axlList * list, int position)
  * return axl_list_lookup (list, __find_item, name);
  *
  * // the lookup function 
- * bool __find_item (axlPointer _element, axlPointer data)
+ * axl_bool __find_item (axlPointer _element, axlPointer data)
  * {
  *	SomeItem * element = _element;
  *	char     * name    = data;
  *
  *	// check the name 
  *	if (axl_cmp (element->name, name))
- *		return true;
+ *		return axl_true;
  *
  *	// it is not the element 
- *	return false;
+ *	return axl_false;
  * }
  * \endcode
  *
@@ -1202,13 +1332,13 @@ axlPointer axl_list_lookup    (axlList * list, axlLookupFunc func, axlPointer da
  * @param data The data provided at \ref axl_list_lookup, in this
  * case, the value we are looking.
  * 
- * @return \ref true if the string was found, \ref false if not.
+ * @return \ref axl_true if the string was found, \ref axl_false if not.
  */
-bool       axl_list_find_string (axlPointer element, axlPointer data)
+axl_bool       axl_list_find_string (axlPointer element, axlPointer data)
 {
-	/* if the string received is null, just return false */
+	/* if the string received is null, just return axl_false */
 	if (data == NULL)
-		return false;
+		return axl_false;
 
 	/* return the comparison status */
 	return axl_cmp ((char *) element, (char *) data);
@@ -1433,16 +1563,16 @@ void            axl_list_cursor_previous (axlListCursor * cursor)
  * @param cursor The cursor that is required to return if there are
  * next items.
  * 
- * @return \ref true if more items are found, otherwise \ref false is
+ * @return \ref axl_true if more items are found, otherwise \ref axl_false is
  * returned.
  */
-bool            axl_list_cursor_has_next (axlListCursor * cursor)
+axl_bool            axl_list_cursor_has_next (axlListCursor * cursor)
 {
-	axl_return_val_if_fail (cursor, false);
+	axl_return_val_if_fail (cursor, axl_false);
 
 	/* check for empty list */
 	if (cursor->current == NULL)
-		return false;
+		return axl_false;
 
 	/* return if the next element isn't null */
 	return (cursor->current->next != NULL);
@@ -1455,16 +1585,16 @@ bool            axl_list_cursor_has_next (axlListCursor * cursor)
  * @param cursor The cursor that is required to return if there are
  * next items.
  * 
- * @return \ref true if more items are found, otherwise \ref false is
+ * @return \ref axl_true if more items are found, otherwise \ref axl_false is
  * returned.
  */
-bool            axl_list_cursor_has_previous (axlListCursor * cursor)
+axl_bool            axl_list_cursor_has_previous (axlListCursor * cursor)
 {
-	axl_return_val_if_fail (cursor, false);
+	axl_return_val_if_fail (cursor, axl_false);
 
 	/* check for empty list */
 	if (cursor->current == NULL)
-		return false;
+		return axl_false;
 
 	/* return if the next element isn't null */
 	return (cursor->current->previous != NULL);
@@ -1476,12 +1606,12 @@ bool            axl_list_cursor_has_previous (axlListCursor * cursor)
  * @param cursor The cursor that is requested to return if a call to
  * \ref axl_list_cursor_get will return data.
  * 
- * @return \ref true if the list that is iterated can return data at
- * the current position, otherwise \ref false is returned.
+ * @return \ref axl_true if the list that is iterated can return data at
+ * the current position, otherwise \ref axl_false is returned.
  */
-bool            axl_list_cursor_has_item    (axlListCursor * cursor)
+axl_bool            axl_list_cursor_has_item    (axlListCursor * cursor)
 {
-	axl_return_val_if_fail (cursor, false);
+	axl_return_val_if_fail (cursor, axl_false);
 
 	/* return if there are current */
 	return (cursor->current != NULL);
@@ -1515,7 +1645,7 @@ void            axl_list_cursor_unlink       (axlListCursor * cursor)
 	cursor->current = (node->next != NULL) ? node->next : node->previous;
 
 	/* call to unlik */
-	__axl_list_common_remove_selected_node (cursor->list, node, false);
+	__axl_list_common_remove_selected_node (cursor->list, node, axl_false);
 
 	return;
 }
@@ -1549,7 +1679,7 @@ void            axl_list_cursor_remove       (axlListCursor * cursor)
 	cursor->current = (node->next != NULL) ? node->next : node->previous;
 
 	/* call to remove */
-	__axl_list_common_remove_selected_node (cursor->list, node, false);
+	__axl_list_common_remove_selected_node (cursor->list, node, axl_true);
 
 	return;
 }
