@@ -12,6 +12,9 @@ static void magnatune_cleanup_xml();
 static axlDoc *magnatune_xmldoc = NULL;
 GMutex *mt_db_lock = NULL;
 
+/* hack fix this */
+int magnatune_end_download();
+
 /**
  * Makes a copy with all &#<id>; decoded
  */
@@ -277,30 +280,34 @@ static void magnatune_cleanup_xml()
     }
 }
 
-void magnatune_db_download_xml(ProgressCallback cb, gpointer data )
-{
 
-    gmpc_easy_download_struct dld = {NULL, 0, -1, cb, data};
+void magnatune_db_download_xml_thread(gpointer data)
+{
+    gmpc_easy_download_struct *dld = data;
+
     g_mutex_lock(mt_db_lock);
 
     if(magnatune_xmldoc)
     {
         axl_doc_free(magnatune_xmldoc);
-        
+
         magnatune_xmldoc = NULL;
     }
-
-    if(gmpc_easy_download("http://www.magnatune.com/info/album_info.xml", &dld))
+    g_mutex_unlock(mt_db_lock);
+    if(gmpc_easy_download("http://www.magnatune.com/info/album_info.xml", dld))
     {
+        g_mutex_lock(mt_db_lock);
         gchar *path = NULL;
-        magnatune_xmldoc = axl_doc_parse(dld.data, dld.size, NULL);
+        magnatune_xmldoc = axl_doc_parse(dld->data, dld->size, NULL);
         magnatune_cleanup_xml();
         path = gmpc_get_user_path("magnatune.xml");
         axl_doc_dump_to_file(magnatune_xmldoc, path); 
         g_free(path);
+        g_mutex_unlock(mt_db_lock);
     }
     else 
     {
+        g_mutex_lock(mt_db_lock);
         /* update */
         gchar *path = gmpc_get_user_path("magnatune.xml");
         if(g_file_test(path, G_FILE_TEST_EXISTS))
@@ -308,12 +315,29 @@ void magnatune_db_download_xml(ProgressCallback cb, gpointer data )
             magnatune_xmldoc = axl_doc_parse_from_file(path, NULL); 
         }
         g_free(path);
+        g_mutex_unlock(mt_db_lock);
     }
     /**
      * cleanup
      */
-    gmpc_easy_download_clean(&dld);
-    g_mutex_unlock(mt_db_lock);
+    gmpc_easy_download_clean(dld);
+    g_free(dld);
+
+
+    
+    g_idle_add(magnatune_end_download, NULL);
+   
+}
+void magnatune_db_download_xml(ProgressCallback cb, gpointer data )
+{
+    gmpc_easy_download_struct *dld = g_malloc0(sizeof(*dld));
+    dld->callback_data = data;
+    dld->callback = cb;
+    dld->max_size = -1;
+    dld->data = NULL;
+    dld->size = 0;
+    g_thread_create(magnatune_db_download_xml_thread, dld,FALSE, NULL);
+
 }
 
 MpdData * magnatune_db_get_artist_list(char *wanted_genre)
