@@ -37,6 +37,9 @@ static void magnatune_show_song_list(GtkTreeSelection *selection, gpointer user_
 static int magnatune_button_handle_release_event_tag_add(GtkWidget *button, gpointer user_data);
 static void magnatune_save_myself(void);
 static GtkTreeRowReference *magnatune_ref = NULL;
+
+
+static int downloading = FALSE; 
 /**
  * Get set enabled 
  */
@@ -61,7 +64,7 @@ static void magnatune_set_enabled(int enabled)
 			GtkTreeIter iter;
             magnatune_save_myself();
 			if (gtk_tree_model_get_iter(GTK_TREE_MODEL(playlist3_get_category_tree_store()), &iter, path)){
-				gtk_tree_store_remove(playlist3_get_category_tree_store(), &iter);
+				gtk_list_store_remove(playlist3_get_category_tree_store(), &iter);
 			}
 			gtk_tree_path_free(path);
 			gtk_tree_row_reference_free(magnatune_ref);
@@ -90,7 +93,7 @@ extern GladeXML *pl3_xml;
 static void magnatune_buy_album()
 {
 	gchar *uri = "http://www.magnatune.com/";
-	gchar *path = g_strdup_printf("%s%c%s%c%s '%s'",plugin.path,G_DIR_SEPARATOR, "magnatune",G_DIR_SEPARATOR,"xdg-open",uri);
+	gchar *path = g_strdup_printf("%s%c%s '%s'",DATA_DIR,G_DIR_SEPARATOR,"xdg-open",uri);
 	g_spawn_command_line_async(path, NULL);
 	g_free(path);
 }
@@ -126,11 +129,12 @@ static void magnatune_logo_add()
 
 static void magnatune_logo_init()
 {
-	gchar *path = NULL;
-
-	path = g_strdup_printf("%s%c%s",plugin.path,G_DIR_SEPARATOR, "magnatune");
-	gtk_icon_theme_append_search_path(gtk_icon_theme_get_default (),path);
-	g_free(path);
+    gchar *path = gmpc_plugin_get_data_path(&plugin);
+    gchar *url = g_build_path(G_DIR_SEPARATOR_S,path, "magnatune", NULL);
+    debug_printf(DEBUG_WARNING,"Found url: %s\n", url);
+	gtk_icon_theme_append_search_path(gtk_icon_theme_get_default (),url);
+    g_free(url);
+    g_free(path);
 
 	gtk_init_add((GtkFunction )magnatune_logo_add, NULL);
 	/** 
@@ -162,7 +166,7 @@ static void   magnatune_mpd_status_changed(MpdObj *mi, ChangedStatusType what, v
 	}
 }
 
-static void magnatune_get_genre_list()
+void magnatune_get_genre_list()
 {
     MpdData *data = NULL;
 	data = magnatune_db_get_genre_list();
@@ -461,7 +465,8 @@ static void magnatune_selected(GtkWidget *container)
 		{
 			magnatune_download();
 		}
-		magnatune_get_genre_list();
+        else
+            magnatune_get_genre_list();
 	} else {
 		gtk_container_add(GTK_CONTAINER(container), magnatune_vbox);
 		gtk_widget_show(magnatune_vbox);
@@ -472,39 +477,67 @@ static void magnatune_unselected(GtkWidget *container)
 	gtk_container_remove(GTK_CONTAINER(container), magnatune_vbox);
 }
 
+int magnatune_end_download()
+{
+    downloading = FALSE;
+
+    magnatune_get_genre_list();
+	gtk_widget_hide(magnatune_pb);
+	gtk_widget_set_sensitive(glade_xml_get_widget(pl3_xml, "pl3_win"), TRUE);
+    return FALSE;
+}
+
+typedef struct _Pass{
+    GtkWidget *pb;
+    int download;
+    int total;
+}Pass;
+
+static int magnatune_download_xml_callback_real(Pass *p);
+/* Move it to main thread, by pushing in idle */
 static void magnatune_download_xml_callback(int download, int total,gpointer data)
 {
-	GtkWidget *pb = data;
+    Pass *p= g_malloc0(sizeof(*p));
+    p->pb = data;
+    p->download = download;
+    p->total =total;
+    g_idle_add(magnatune_download_xml_callback_real, p);
+}
+
+static int magnatune_download_xml_callback_real(Pass *p)
+{
+	GtkWidget *pb = p->pb;
+    int download = p->download;
+    int total = p->total;
 	gchar *label = NULL;
+    gchar *size = NULL;
+//    gdk_threads_enter();
 	if(total > 0)
 		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(pb), download/(float)total);
 	else
 		gtk_progress_bar_pulse(GTK_PROGRESS_BAR(pb));
-	if(download >> 20)
-	{
-		label = g_strdup_printf("%.2f mb",(download>>10)/1024.0);
-	}
-	else if (download>>10)
-	{
-		label = g_strdup_printf("%i kb",download>>10);
-	}
-	else
-	{
-		label = g_strdup_printf("%i b",download);
-	}
-	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(pb), label);
+
+    size = g_format_size_for_display((goffset)download);
+    label = g_strdup_printf("Downloading music catalog (%s done)",size);
+    g_free(size);
+    gtk_progress_bar_set_text(GTK_PROGRESS_BAR(pb), label);
 	g_free(label);
-	while(gtk_events_pending())
-		gtk_main_iteration();
+//	while(gtk_events_pending())
+//		gtk_main_iteration();
+//    gdk_threads_leave();
+    g_free(p);
+    return FALSE;
 }
 
 static void magnatune_download()
 {
-	gtk_widget_set_sensitive(glade_xml_get_widget(pl3_xml, "pl3_win"), FALSE);
+    downloading = TRUE; 
+    //	gtk_widget_set_sensitive(glade_xml_get_widget(pl3_xml, "pl3_win"), FALSE);
+    gmpc_mpddata_model_set_mpd_data(GMPC_MPDDATA_MODEL(gtk_tree_view_get_model(GTK_TREE_VIEW(treeviews[0]))),NULL);
 	gtk_widget_show(magnatune_pb);
 	magnatune_db_download_xml(magnatune_download_xml_callback, magnatune_pb);
-	gtk_widget_hide(magnatune_pb);
-	gtk_widget_set_sensitive(glade_xml_get_widget(pl3_xml, "pl3_win"), TRUE);
+//	gtk_widget_hide(magnatune_pb);
+//	gtk_widget_set_sensitive(glade_xml_get_widget(pl3_xml, "pl3_win"), TRUE);
 }
 
 static void magnatune_add(GtkWidget *cat_tree)
@@ -518,8 +551,7 @@ static void magnatune_add(GtkWidget *cat_tree)
 
     debug_printf(DEBUG_INFO,"Adding at position: %i", pos);
 	playlist3_insert_browser(&iter, pos);
-	/*gtk_tree_store_append(pl3_tree, &iter, NULL);*/
-	gtk_tree_store_set(pl3_tree, &iter, 
+	gtk_list_store_set(pl3_tree, &iter, 
 			PL3_CAT_TYPE, plugin.id,
 			PL3_CAT_TITLE, "Magnatune Browser",
 			PL3_CAT_INT_ID, "",
@@ -575,6 +607,9 @@ static char * __magnatune_process_string(char *name)
 static int magnatune_fetch_cover_priority(void){
 	return cfg_get_single_value_as_int_with_default(config, "magnatune", "priority", 80);
 }
+static void magnatune_fetch_cover_priority_set(int priority){
+	cfg_set_single_value_as_int(config, "magnatune", "priority", priority);
+}
 static int magnatune_fetch_get_image(mpd_Song *song,MetaDataType type, char **path)
 {
 	int result = 0;
@@ -586,11 +621,8 @@ static int magnatune_fetch_get_image(mpd_Song *song,MetaDataType type, char **pa
 		char *value = magnatune_db_get_value(artist,NULL,META_ARTIST_ART);
 		if(value)
 		{
-			gchar *filename = NULL;
 			gmpc_easy_download_struct dld = {NULL, 0, -1, NULL,NULL};
-			filename = g_strdup_printf("%s.jpg", artist);
-			*path = gmpc_get_covers_path(filename);
-			g_free(filename); 
+			*path = gmpc_get_metadata_filename(META_ARTIST_ART, song, NULL); 
 			if(gmpc_easy_download(value, &dld))
 			{
 				g_file_set_contents(*path, dld.data, dld.size, NULL);
@@ -609,11 +641,9 @@ static int magnatune_fetch_get_image(mpd_Song *song,MetaDataType type, char **pa
 		char *value = magnatune_db_get_value(artist,album,META_ALBUM_ART);
 		if(value)
 		{
-			gchar *filename;
 			gmpc_easy_download_struct dld = {NULL, 0, -1, NULL,NULL};
-			filename = g_strdup_printf("%s-%s.jpg", artist,album);
-			*path = gmpc_get_covers_path(filename);
-			g_free(filename); 
+
+			*path = gmpc_get_metadata_filename(META_ALBUM_ART, song, NULL); 
 			if(gmpc_easy_download(value, &dld))
 			{
 				g_file_set_contents(*path, dld.data, dld.size, NULL);
@@ -637,10 +667,10 @@ static void magnatune_redownload_reload_db()
 		GtkTreeIter citer;
 		while(gtk_tree_model_iter_children(model, &citer,&iter))
 		{
-			gtk_tree_store_remove(GTK_TREE_STORE(model), &citer);
+			gtk_list_store_remove(GTK_LIST_STORE(model), &citer);
 		}
 		magnatune_download();
-		magnatune_get_genre_list();
+		//magnatune_get_genre_list();
 	}
 	if(path)
 		gtk_tree_path_free(path);
@@ -723,7 +753,7 @@ static int magnatune_cat_menu_popup(GtkWidget *menu, int type, GtkWidget *tree,
 {
 	GtkWidget *item;
 	if(type != plugin.id) return 0;
-	else
+	else if (!downloading)
     {
         /* add the clear widget */
         item = gtk_image_menu_item_new_from_stock(GTK_STOCK_REFRESH,NULL);
@@ -971,8 +1001,9 @@ static void magnatune_destroy(void)
 }
 
 gmpcMetaDataPlugin magnatune_cover = {
-	magnatune_fetch_cover_priority,
-	magnatune_fetch_get_image
+	.get_priority = magnatune_fetch_cover_priority,
+    .set_priority = magnatune_fetch_cover_priority_set,
+	.get_image = magnatune_fetch_get_image
 };
 
 
